@@ -9,6 +9,8 @@ import sys
 import signal
 from osc import oscerr
 from urllib2 import URLError, HTTPError
+# import as RPMError because the class "error" is too generic
+from rpm import error as RPMError
 
 # the good things are stolen from Matt Mackall's mercurial
 
@@ -62,6 +64,14 @@ def run(prg):
         print >>sys.stderr, 'aborted.'
         return 1
 
+    except oscerr.APIError, e:
+        print >>sys.stderr, 'BuildService API error:', e.msg
+        return 1
+
+    except oscerr.LinkExpandError, e:
+        print >>sys.stderr, 'Link cannot be expanded:\n', e
+        return 1
+
     except oscerr.UnreadableFile, e:
         print >>sys.stderr, e.msg
         return 1
@@ -81,10 +91,11 @@ def run(prg):
                 print >>sys.stderr, e.hdrs
                 print >>sys.stderr, body
 
-        if e.code in [ 400, 403, 404 ]:
-            msg = body.split('<summary>')[1]
-            msg = msg.split('</summary>')[0]
-            print >>sys.stderr, msg
+        if e.code in [ 400, 403, 404, 500 ]:
+            if '<summary>' in body:
+                msg = body.split('<summary>')[1]
+                msg = msg.split('</summary>')[0]
+                print >>sys.stderr, msg
 
         return 1
 
@@ -122,3 +133,7 @@ def run(prg):
     except AttributeError, e:
         print >>sys.stderr, e
         return 1
+    
+    except RPMError, e:
+        print >>sys.stderr, e
+        return 1
