@@ -1,4 +1,5 @@
 #!/usr/bin/python
+# vim: sw=4 et
 
 # Copyright (C) 2006 Peter Poeml / Novell Inc.  All rights reserved.
 # This program is free software; it may be used, copied, modified
@@ -405,6 +406,7 @@ class Osc(cmdln.Cmdln):
     @cmdln.option('--nodevelproject', action='store_true',
                   help='do not follow a defined devel project ' \
                        '(primary project where a package is developed)')
+    @cmdln.alias("sr")
     def do_submitreq(self, subcmd, opts, *args):
         """${cmd_name}: Handle requests to submit a package into another project
 
@@ -430,6 +432,9 @@ class Osc(cmdln.Cmdln):
         "decline" will change the request state to "declined" and append a
         message that you specify with the --message option.
 
+        "delete" will change the request state to "deleted" and append a
+        message that you specify with the --message option.
+
         "accept" will change the request state to "accepted" and will trigger
         the actual submit process. That would normally be a server-side copy of
         the source package to the target package.
@@ -438,16 +443,17 @@ class Osc(cmdln.Cmdln):
         usage:
             osc submitreq create [-m TEXT] 
             osc submitreq create [-m TEXT] SOURCEPRJ SOURCEPKG DESTPRJ [DESTPKG]
-            osc submitreq list PRJ [PKG]
+            osc submitreq list [PRJ [PKG]]
             osc submitreq show [-d] ID
             osc submitreq decline [-m TEXT] ID
+            osc submitreq delete [-m TEXT] ID
             osc submitreq accept [-m TEXT] ID
         ${cmd_option_list}
         """
 
         args = slash_split(args)
 
-        cmds = ['create', 'list', 'show', 'decline', 'accept']
+        cmds = ['create', 'list', 'show', 'decline', 'accept', 'delete']
         if not args or args[0] not in cmds:
             raise oscerr.WrongArgs('Unknown submitreq action. Choose one of %s.' \
                                                % ', '.join(cmds))
@@ -458,7 +464,7 @@ class Osc(cmdln.Cmdln):
         if cmd in ['create']:
             min_args, max_args = 0, 4
         elif cmd in ['list']:
-            min_args, max_args = 1, 2
+            min_args, max_args = 0, 2
         else:
             min_args, max_args = 1, 1
         if len(args) < min_args:
@@ -490,20 +496,32 @@ class Osc(cmdln.Cmdln):
                     dst_package = args[3]
                 else:
                     dst_package = src_package
+            else:
+                raise oscerr.WrongArgs('Incorrect number of arguments.\n\n' \
+                      + self.get_cmd_help('submitreq'))
 
         elif cmd == 'list':
-            project = args[0]
+            package = None
+            project = None
+            if len(args) > 0:
+                project = args[0]
+            else:
+                project = store_read_project(os.curdir)
+                apiurl = store_read_apiurl(os.curdir)
+                try:
+                    package = store_read_package(os.curdir)
+                except oscerr.NoWorkingCopy:
+                    pass
+
             if len(args) > 1:
                 package = args[1]
-            else:
-                package = None
-        elif cmd in ['show', 'decline', 'accept']:
+        elif cmd in ['show', 'decline', 'accept', 'delete']:
             reqid = args[0]
 
 
         # create
         if cmd == 'create':
-            devloc = show_develproject(conf.config['apiurl'], dst_project, dst_package)
+            devloc = show_develproject(apiurl, dst_project, dst_package)
             if devloc \
                     and dst_project != devloc \
                     and src_project != devloc \
@@ -524,8 +542,11 @@ Please submit there instead, or use --nodevelproject to force direct submission.
 
         # list
         elif cmd == 'list':
-            results = get_submit_request_list(conf.config['apiurl'], 
+            results = get_submit_request_list(apiurl, 
                                              project, package)
+
+            results.sort(reverse=True)
+
             for result in results:
                 print result.list_view()
 
@@ -555,6 +576,11 @@ Please submit there instead, or use --nodevelproject to force direct submission.
             r = change_submit_request_state(conf.config['apiurl'], 
                     reqid, 'accepted', opts.message or '')
             print r
+        # delete
+        elif cmd == 'delete':
+            r = change_submit_request_state(conf.config['apiurl'],
+                    reqid, 'deleted', opts.message or '')
+            print r
 
 
     # editmeta and its aliases are all depracated
@@ -579,6 +605,8 @@ Please submit there instead, or use --nodevelproject to force direct submission.
         return 2
 
 
+    @cmdln.option('-r', '--revision', metavar='rev',
+                  help='link the specified revision.')
     def do_linkpac(self, subcmd, opts, *args):
         """${cmd_name}: "Link" a package to another package
         
@@ -606,6 +634,8 @@ Please submit there instead, or use --nodevelproject to force direct submission.
             raise oscerr.WrongArgs('Incorrect number of arguments.\n\n' \
                   + self.get_cmd_help('linkpac'))
 
+        rev, dummy = parseRevisionOption(opts.revision)
+
         src_project = args[0]
         src_package = args[1]
         dst_project = args[2]
@@ -617,7 +647,12 @@ Please submit there instead, or use --nodevelproject to force direct submission.
         if src_project == dst_project and src_package == dst_package:
             print >>sys.stderr, 'Error: source and destination are the same.'
             return 1
-        link_pac(src_project, src_package, dst_project, dst_package)
+
+        if rev and not checkRevision(src_project, src_package, rev):
+            print >>sys.stderr, 'Revision \'%s\' does not exist' % rev
+            sys.exit(1)
+
+        link_pac(src_project, src_package, dst_project, dst_package, rev)
 
     def do_aggregatepac(self, subcmd, opts, *args):
         """${cmd_name}: "Aggregate" a package to another package
@@ -1272,6 +1307,13 @@ Please submit there instead, or use --nodevelproject to force direct submission.
                 rev = show_upstream_xsrcmd5(p.apiurl,
                                             p.prjname, p.name)
 
+            # FIXME: ugly workaround for #399247
+            if opts.expand_link or opts.unexpand_link:
+                if [ i for i in p.filenamelist+p.filenamelist_unvers if p.status(i) != ' ' ]:
+                    print >>sys.stderr, 'osc: cannot expand/unexpand because your working ' \
+                                        'copy has local modifications. Please remove them ' \
+                                        'and try again'
+                    sys.exit(1)
             p.update(rev)
             rev = None
                    
@@ -1400,6 +1442,7 @@ Please submit there instead, or use --nodevelproject to force direct submission.
             print ''.join(show_results_meta(pac.apiurl, pac.prjname, package=pac.name))
 
                 
+    @cmdln.alias('r')
     def do_results(self, subcmd, opts, *args):
         """${cmd_name}: Shows the build results of a package
 
@@ -1440,6 +1483,7 @@ Please submit there instead, or use --nodevelproject to force direct submission.
     @cmdln.option('-n', '--name-filter', metavar='EXPR',
                         help='show only packages whos name matches EXPR')
                        
+    @cmdln.alias('pr')
     def do_prjresults(self, subcmd, opts, *args):
         """${cmd_name}: Shows project-wide build results
         
@@ -1939,6 +1983,8 @@ Please submit there instead, or use --nodevelproject to force direct submission.
                         help='search for matches in the \'description\' element')
     @cmdln.option('-v', '--verbose', action='store_true',
                         help='show more information')
+    @cmdln.option('-i', '--involved', action='store_true',
+                        help='show involved projects/packages')
     def do_search(self, subcmd, opts, *args):
         """${cmd_name}: Search for a project and/or package.
 
@@ -1951,10 +1997,17 @@ Please submit there instead, or use --nodevelproject to force direct submission.
         ${cmd_option_list}
         """
 
+        search_term = None
         if len(args) > 1:
             raise oscerr.WrongArgs('Too many arguments.')
-        elif len(args) < 1:
+        elif len(args) < 1 and not opts.involved:
             raise oscerr.WrongArgs('Too few arguments.')
+        elif len(args) == 1:
+            search_term = args[0]
+
+        if (opts.title or opts.description) and opts.involved:
+            raise oscerr.WrongArgs('Sorry, the options \'--title\' and/or \'--description\' ' \
+                                   'plus \'--involved\' are mutual exclusive')
 
         search_list = []
         search_for = []
@@ -1968,13 +2021,17 @@ Please submit there instead, or use --nodevelproject to force direct submission.
         if opts.project:
             search_list.append('@name')
             search_for.append('project')
+        if opts.involved:
+            search_list = [ 'person/@userid' ]
+            search_term = search_term or conf.get_apiurl_usr(conf.config['apiurl'])
+            opts.enable_exact = True
 
         if not search_list:
             search_list = ['title', 'description', '@name']
         if not search_for:
             search_for = [ 'project', 'package' ]
         for kind in search_for:
-            result = search(conf.config['apiurl'], set(search_list), kind, args[0], opts.verbose, opts.enable_exact, opts.repos_baseurl)
+            result = search(conf.config['apiurl'], set(search_list), kind, search_term, opts.verbose, opts.enable_exact, opts.repos_baseurl)
             if result:
                 if kind == 'package':
                     headline = [ '# Package', '# Project' ]
@@ -1986,11 +2043,11 @@ Please submit there instead, or use --nodevelproject to force direct submission.
                     headline.append('# URL')
                 if len(search_for) > 1:
                     print '#' * 68
-                print 'matches for \'%s\' in %ss:\n' % (args[0], kind)
+                print 'matches for \'%s\' in %ss:\n' % (search_term, kind)
                 for row in build_table(len(headline), result, headline, 2):
                     print row
             else:
-               print 'No matches found for \'%s\' in %ss' % (args[0], kind)
+               print 'No matches found for \'%s\' in %ss' % (search_term, kind)
 
 
     @cmdln.option('-p', '--project', metavar='project',
